import Flutter
import Foundation

open class FlCameraMethodCall: NSObject {
    public var flCameraEvent: FlCameraEvent?
    var flCameraChannel: FlutterMethodChannel?
    public var registrar: FlutterPluginRegistrar?
    public var flCamera: FlCamera?

    public init(_ _registrar: FlutterPluginRegistrar) {
        super.init()
        registrar = _registrar
    }

    open func handle(call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "initCamera":
            if flCamera == nil, flCameraEvent != nil, registrar != nil {
                flCamera = FlCamera(flCameraEvent!, registrar!.textures())
            }
            result(flCamera != nil)
        case "startPreview":
            startPreview(nil, call: call, result)
        case "setFlashMode":
            flCamera?.setFlashMode(status: call.arguments as! Bool)
            result(flCamera != nil)
        case "disposeCamera":
            flCamera?.close()
            flCamera = nil
            result(flCamera == nil)
        case "startEvent":
            if flCameraEvent == nil, registrar != nil {
                flCameraEvent = FlCameraEvent(registrar!.messenger())
            }
            result(flCameraEvent != nil)
        case "sendEvent":
            flCameraEvent?.sendEvent(call.arguments)
            result(flCameraEvent != nil)
        case "stopEvent":
            disposeEvent()
            result(flCameraEvent == nil)
        default:
            result(FlutterMethodNotImplemented)
        }
    }

    open func startPreview(_ captureOutputCallBack: ((_ buffer: CMSampleBuffer) -> Void)?, call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        let arguments = call.arguments as! [String: Any?]
        let useBackCamera = arguments["useBackCamera"] as! Bool
        let zoom = arguments["zoom"] as! Int
        if flCamera == nil { result(nil) }
        flCamera?.initCamera(useBackCamera, zoom, result, captureOutputCallBack)
    }

    func disposeEvent() {
        if flCameraEvent != nil {
            flCameraEvent!.dispose()
            flCameraEvent = nil
        }
    }

    func dispose() {
        disposeEvent()
        flCamera?.close()
        flCamera = nil
        flCameraChannel = nil
        registrar = nil
    }
}
