part of '../fl_camera.dart';

typedef EventListen = void Function(dynamic data);

class FLCameraEvent {
  factory FLCameraEvent() => _getInstance();

  FLCameraEvent._internal();

  static FLCameraEvent get instance => _getInstance();
  static FLCameraEvent? _instance;

  static FLCameraEvent _getInstance() {
    _instance ??= FLCameraEvent._internal();
    return _instance!;
  }

  /// 订阅流
  /// Subscription flow
  StreamSubscription<dynamic>? _streamSubscription;

  /// 创建流
  /// Create stream
  Stream<dynamic>? _stream;

  MethodChannel _channel = _flCameraChannel;

  /// 消息通道
  /// Message channel
  EventChannel? _eventChannel;

  MethodChannel get channel => _channel;

  EventChannel? get eventChannel => _eventChannel;

  /// 第一步 先设置 channel
  /// The first step is to set the channel
  void setMethodChannel(MethodChannel channel) {
    _channel = channel;
  }

  bool get isPaused =>
      _streamSubscription != null && _streamSubscription!.isPaused;

  /// 第二步 初始化消息通道
  /// Step 2: initialize the message channel
  Future<bool> initialize() async {
    if (!_supportPlatform) return false;
    bool? state = await _channel.invokeMethod<bool?>('startEvent');
    state ??= false;
    if (state && _eventChannel == null) {
      _eventChannel = const EventChannel(_flCameraEvent);
      _stream = _eventChannel?.receiveBroadcastStream(<dynamic, dynamic>{});
    }
    return state && _eventChannel != null && _stream != null;
  }

  /// 第三步 添加消息流监听
  /// Step 3: add message flow listening
  Future<bool> addListener(EventListen eventListen) async {
    if (!_supportPlatform) return false;
    if (_eventChannel != null && _stream != null) {
      try {
        _streamSubscription = _stream!.listen(eventListen);
        return true;
      } catch (e) {
        print(e);
        return false;
      }
    }
    return false;
  }

  /// 调用原生方法 发送消息
  /// Call native methods to send messages
  Future<bool> sendEvent(dynamic arguments) async {
    if (!_supportPlatform ||
        _eventChannel == null ||
        _streamSubscription == null ||
        _streamSubscription!.isPaused) return false;
    final bool? state =
    await _channel.invokeMethod<bool?>('sendEvent', arguments);
    return state ?? false;
  }

  /// 暂停消息流监听
  /// Pause message flow listening
  bool pause() {
    if (!_supportPlatform) return false;
    if (_streamSubscription != null && !_streamSubscription!.isPaused) {
      _streamSubscription!.pause();
      return true;
    }
    return false;
  }

  /// 重新开始监听
  /// Restart listening
  bool resume() {
    if (!_supportPlatform) return false;
    if (_streamSubscription != null && _streamSubscription!.isPaused) {
      _streamSubscription!.resume();
      return true;
    }
    return false;
  }

  /// 关闭并销毁消息通道
  /// Close and destroy the message channel
  Future<bool> dispose() async {
    if (!_supportPlatform) return false;
    await _streamSubscription?.cancel();
    _streamSubscription = null;
    _stream = null;
    _eventChannel = null;
    final bool? state = await _channel.invokeMethod<bool>('stopEvent');
    return state ?? false;
  }
}
