part of '../fl_camera.dart';

enum ZoomQuality { high, low }

class FlCameraMethodCall {
  factory FlCameraMethodCall() => _getInstance();

  FlCameraMethodCall._internal();

  static FlCameraMethodCall get instance => _getInstance();
  static FlCameraMethodCall? _instance;

  static FlCameraMethodCall _getInstance() {
    _instance ??= FlCameraMethodCall._internal();
    return _instance!;
  }

  MethodChannel _channel = _flCameraChannel;
  FlCameraOptions? _cameraOptions;

  MethodChannel get channel => _channel;

  FlCameraOptions? get cameraOptions => _cameraOptions;

  /// 第一步 先设置 channel
  /// The first step is to set the channel
  void setMethodChannel(MethodChannel channel) {
    _channel = channel;
  }

  /// 第二步 初始化相机
  /// Step 2 initialize the camera
  Future<bool> initCamera() async {
    if (!_supportPlatform) return false;
    final bool? state = await channel.invokeMethod<bool?>('initCamera');
    return state ?? false;
  }

  /// 第三步 开始预览
  /// Step 3 start Preview
  Future<FlCameraOptions?> startPreview({bool useBackCamera = true,
    ZoomQuality zoomQuality = ZoomQuality.low}) async {
    if (!_supportPlatform) return null;
    final Map<dynamic, dynamic>? map = await channel
        .invokeMethod<Map<dynamic, dynamic>?>('startPreview', <String, dynamic>{
      'useBackCamera': useBackCamera,
      'zoom': ZoomQuality.values.indexOf(zoomQuality) + 1
    });
    if (map != null) {
      _cameraOptions = FlCameraOptions.fromMap(map);
      return _cameraOptions;
    }
    return null;
  }

  /// 打开\关闭 闪光灯
  /// Turn flash on / off
  Future<bool> setFlashMode(bool status) async {
    if (!_supportPlatform) return false;
    final bool? state =
    await channel.invokeMethod<bool?>('setFlashMode', status);
    return state ?? false;
  }

  /// 注销相机
  Future<bool> dispose() async {
    if (!_supportPlatform) return false;
    final bool? state = await channel.invokeMethod<bool?>('disposeCamera');
    return state ?? false;
  }
}

class FlCameraOptions {
  FlCameraOptions.fromMap(Map<dynamic, dynamic> map) {
    textureId = map['textureId'] as int?;
    width = map['width'] as double?;
    height = map['height'] as double?;
  }

  int? textureId;
  double? width;
  double? height;

  Map<String, dynamic> toMap() =>
      <String, dynamic>{
        'textureId': textureId,
        'width': width,
        'height': height
      };
}
